function [ CPMatric, CINumber ] = Pearl_PolytreeRecover( CIMatric,CPMatricSkeleton )
%The recovery algorithm consists of the following steps:
%1 Generate a maximum-weight undirected spanning tree using the MWST
%2 Search the internal nodes of the skeleton, beginning with the outermost 
%  layer and working inward, until a multi-parent node is found using the 
%  X?Y?Z test described at the beginning of the section.
%3.When a multi-parent node n is found, determine the directionality of all
%  of its branches using the X?Y?Z test.
%4.For each node having at least incoming arrow, resolve the directionality
%  of all of its remaining adjacent branches using the X?Y?Z test.
%5 Repeat steps 2 through 4 until no further directionality can be discovered.

%Step 1.
%Take input variable CPMatric as the MWST as the one proposed in Step one.

CPMatric = CPMatricSkeleton;
Dim = size( CPMatric,2 );
CINumber = 0;
ArticulationPointSet = zeros(1,Dim);

%Step 2.
% We take all inward nodes out, among those nodes, we manage to make a 
% topological searching from the outermost layer toward inward layer by
% BFS using "queue" data structure.

% Find the leaf "p" of polytree skeleton on the outmost layer.
for p=1:Dim
    if IsLeaf( CPMatric, p) == 1
        q = find( CPMatric(p,:)==1,1);
        break;
    end
end

%Initiate the queue, and start the searching process.
Queue = zeros( 1,Dim );
n=1;
Front = 1; End = 2;
Queue(1) = q;
VisitedNodes = zeros( 1,Dim );
% h3 = view(biograph( CPMatric ))
while Front ~= End
      CurrentNode = Queue( Front );
      VisitedNodes( CurrentNode ) = 1;
      Front = QueueMove( Front,Dim );  
      
     % Update the waiting nodes in "Queue" that should be searched 
     for p = 1:Dim
         if ( CPMatric( p,CurrentNode ) == 1 || CPMatric( CurrentNode,p ) == 1) && IsLeaf( CPMatric,p )==0  && VisitedNodes( p )==0           
            Queue( End ) = p;              
            End = QueueMove( End,Dim ); 
         end  
     end     
      
      % In this section we determine whether "VisitedNode" is multi-parent or
      % not, and we further the orientation once VisitedNode is multi-parent by 
      % the orienting principle : 1. X-> Y <- Z ; 2. X->Y->Z  
            
      % 1. p -- V* -- q ==> p -> V* <- q
      Oriented = 0;
      for p = 1: ( Dim-1 )
          if  CPMatric( CurrentNode,p ) == 1 && CPMatric( p,CurrentNode ) == 1 
              for q = p+1:Dim 
                  if  CPMatric( CurrentNode,q ) == 1 && CPMatric( q,CurrentNode ) == 1 
                      CINumber = CINumber + 1; 
                      if CIMatric( p,q ) == 1 
                         CPMatric( CurrentNode,q )= 0 ; 
                         CPMatric( CurrentNode,p )= 0 ;
                         Oriented = 1;
                      end                
                  end                  
              end                   
          end               
      end
      % 2. p -> V* -- q ==> p -> V* -> q   
      p=1;
      while p <= Dim         
         if  CPMatric( p,CurrentNode ) == 1 && CPMatric( CurrentNode,p ) == 0     
             break;
         end
         p = p + 1;
      end
      if p<= Dim
         for q = 1:Dim
            if q ~= p && CPMatric( q,CurrentNode ) == 1 && CPMatric( CurrentNode,q ) == 1
               CPMatric( q,CurrentNode ) = 0;
               CPMatric( CurrentNode,q ) = 1;
            end         
         end     
      end
      
     % If the current node has satisfied the orienting principle 1.           
     if Oriented == 1
        ArticulationPointSet( n ) = CurrentNode; 
        n = n + 1;
     end
           
end
end

function Next = QueueMove( Current,Dim )
     Next = mod( Current,Dim+1 )+1 ;
end